/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - SQL05 for C using RDMSQL */
/* When you run this example, a new database is created with a two tables
 * that need to be related via a many-to-many relationship, so an
 * 'intersect' table is also created to keep track of that information.
 * Three rows are inserted into each of the main tables and for each
 * relationship a row is inserted into the intersect table.
 * Finally all the data is read starting from one main table and printed
 * and then the data is reread and printed starting from the other main
 * table.
 *
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDB/rsqlCloseDB
 *      rsqlExecDirect
 *      rsqlFetch
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown execptions
 * but good programming practices would dictate that this occur in
 * a full application.
 */

static void create_database(void)
{
    RSQL_ERRCODE rsqlCode;
    HCONN        hDbc;
    HSTMT        hStmt;
    int16_t      ii;
    const char  *cmds[] = {
        "CREATE DATABASE SQL05_db",
        "CREATE TABLE MainTab1 (id INTEGER PRIMARY KEY, str CHAR(30))",
        "CREATE TABLE MainTab2 (id INTEGER PRIMARY KEY, str CHAR(30))",
        "CREATE TABLE IntTab (id1 INTEGER REFERENCES MainTab1(id), id2 INTEGER REFERENCES MainTab2(id))",
        "COMMIT"
    };

    (void) rsqlAllocConn(&hDbc);
    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    (void) rsqlExecDirect(hStmt, "DROP DATABASE SQL05_db");
    (void) rsqlExecDirect(hStmt, "COMMIT");

    for (ii = 0; ii < sizeof(cmds) / sizeof(cmds[0]); ii++)
    {
        rsqlCode = rsqlExecDirect(hStmt, cmds[ii]);
        if (rsqlCode != errSUCCESS)
            break;
    }

    if (rsqlCode != errSUCCESS)
        fprintf(stderr, "Error creating the SQL05 database\n");

    /* Close the newly created database */
    (void) rsqlCloseDB(hDbc, "SQL05_db");

    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);
}

/* =======================================================================
*/
int32_t main(void)
{
    RSQL_ERRCODE      rsqlCode;
    HCONN             hDbc;
    HSTMT             hStmt;
    const RSQL_VALUE *result;
    uint16_t          numCols;
    char              commandText[80];
    int16_t           ii, jj;

    create_database();

    /* Allocate handles */
    (void) rsqlAllocConn(&hDbc);
    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Open the database in exclusive mode */
    (void) rsqlOpenDB(hDbc, "SQL05_db", "x");

    /* Insert three rows into the first main table */
    for (ii = 1; ii < 4; ii++)
    {
        sprintf(commandText, "INSERT INTO MainTab1 VALUES %hd, 'main1 %hd'", ii, ii);
        rsqlCode = rsqlExecDirect(hStmt, commandText);
    }

    /* Insert three rows into the first main table */
    for (ii = 1; ii < 4; ii++)
    {
        sprintf(commandText, "INSERT INTO MainTab2 VALUES %hd, 'main2 %hd'", ii, ii);
        rsqlCode = rsqlExecDirect(hStmt, commandText);
    }

    /* Insert a row into the intersect table for each combination of
     * rows in the main tables */
    for (ii = 1; ii < 4; ii++)
    {
        for (jj = 1; jj < 4; jj++)
        {
            sprintf(commandText, "INSERT INTO IntTab VALUES %hd, %hd", ii, jj);
            rsqlCode = rsqlExecDirect(hStmt, commandText);
        }
    }

    /* Scan through the relationships from the first main table to the second */
    printf("From MainTab1 to MainTab2\n");
    (void) rsqlExecDirect(hStmt, "SELECT m1.str, m2.str FROM MainTab1 m1, MainTab2 m2, IntTab WHERE m1.id = IntTab.id1 AND m2.id = IntTab.id2");

    while ((rsqlCode = rsqlFetch(hStmt, &result, &numCols)) == errSUCCESS)
        printf("%s -- %s\n", result[0].vt.cv, result[1].vt.cv);

    /* Close the database */
    (void) rsqlCloseDB(hDbc, "SQL05_db");

    /* Free handles */
    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}

